#include "samplela.h"
#include <string>
#include <stdlib.h>
#include <mainsimu.h>
#include <message.h>
#include "aux.h"


static const unsigned NUM_CTL_IN_PORT	=8;
static const unsigned NUM_CTL_OUT_PORT	=7;
static const unsigned NUM_ADDR_PORT		=1;
static const unsigned NUM_BSEL_PORT		=1;
static const unsigned NUM_DATA_PORT		=1;

static const string ctlInPortNames[] ={"busyin","dtackin","errin","rwin","resetin","asin","bgrantin","cs"};
static const string ctlOutPortNames[]={"busyout", "dtackout","errout","rwout","resetout","asout", "bgrantout" };
static const string addrInPortNames[]={ "ain" };
static const string addrOutPortNames[]	={"aout"};
static const string bselInPortNames[]	={"bselin"};
static const string bselOutPortNames[]	={"bselout"};
static const string dataInPortNames[]	={"dbusin"};
static const string dataOutPortNames[]	={"dbusout"};


	
sampleLA::sampleLA( const string &name)
	: Atomic (name)
	, m_responseTime ( 0, 0, 0, 1)
{

	//Gathering model parameters

	string time( MainSimulator::Instance().getParameter( this->description(), "response" ));

	string minAddrSpaceLimit( MainSimulator::Instance().getParameter( this->description(), "minAddr" ));
	string maxAddrSpaceLimit( MainSimulator::Instance().getParameter( this->description(), "maxAddr" ));
	
	m_minAddrSpaceLimit = stolli ( minAddrSpaceLimit );
	m_maxAddrSpaceLimit = stolli ( maxAddrSpaceLimit );
	
	//Read the parameters
	if( time != "" )
		m_responseTime = time ;

	

	//Add the ports

	m_ctlInPorts=addPorts(	ptInput, ctlInPortNames, 
							NUM_CTL_IN_PORT, *this); 

	m_ctlOutPorts=addPorts(	ptOutput, ctlOutPortNames, 
							NUM_CTL_OUT_PORT, *this);

 	m_addrInPorts = addPorts(	ptInput,addrInPortNames[0],
 								m_addrWidth,*this);

	m_addrOutPorts = addPorts(	ptOutput,addrOutPortNames[0],
								m_addrWidth,*this);
								
	m_bselInPorts = addPorts(	ptInput,bselInPortNames[0],
								m_bselWidth,*this);

	m_bselOutPorts = addPorts(	ptOutput,bselOutPortNames[0],
								m_bselWidth,*this);

	m_dataInPorts = addPorts(	ptInput,dataInPortNames[0],
								m_dataWidth,*this);

	m_dataOutPorts = addPorts(	ptOutput,dataOutPortNames[0],
								m_dataWidth,*this);

	//Initializing the internal memory cells of the model
	m_ctlIn=new (bit) [NUM_CTL_IN_PORT];
	m_ctlOut=new (o_bit) [NUM_CTL_OUT_PORT];
	m_addrIn=new (bit) [m_addrWidth];
	m_addrOut=new (o_bit) [m_addrWidth];
	m_bselIn=new (bit) [m_bselWidth];
	m_bselOut=new (o_bit) [m_bselWidth];
	m_dataIn=new (bit) [m_dataWidth];
	m_dataOut=new (o_bit) [m_dataWidth];
	
	m_state=waiting;

}

sampleLA::~sampleLA(){

	delete[] m_ctlInPorts;
	delete[] m_ctlOutPorts;
	delete[] m_addrInPorts;
	delete[] m_addrOutPorts;
	delete[] m_bselInPorts;
	delete[] m_bselOutPorts;
	delete[] m_dataInPorts;
	delete[] m_dataOutPorts;

	delete[] m_ctlIn;
	delete[] m_ctlOut;
	delete[] m_addrIn;
	delete[] m_addrOut;
	delete[] m_bselIn;
	delete[] m_bselOut;
	delete[] m_dataIn;
	delete[] m_dataOut;

}

Model & sampleLA::initFunction()
{
	
	m_ctlIn[resetin]=1;
	
	m_ctlOut[bgrantout].val=1;
	m_ctlOut[resetout].val=1;
	m_ctlOut[busyout].val=0;
	m_ctlOut[dtackout].val=0;
	
    holdIn( active, Time::Zero ); 
	
	return *this;
}

Model & sampleLA::externalFunction( const ExternalMessage & msg)
{

 	string portName;
	unsigned long long    portNum;
	unsigned i;

	nameNum(msg.port().name(), portName, portNum );
	
	for( i=0; i<NUM_CTL_IN_PORT; i++)
		if ( portName == ctlInPortNames[i] )
			m_ctlIn[i]= bit (msg.value());

	if ( portName == string(addrInPortNames[0]) ){
		m_addrIn[portNum]= bit (msg.value());
	}

	if (portName == bselInPortNames[0] )
		m_bselIn[portNum]= bit (msg.value());

	if (portName == dataInPortNames[0] )
		m_dataIn[portNum]= bit (msg.value());


	//This component will only output all Data bits in 1 when someone tries to read from its address space

	if (m_ctlIn[asin] && (m_state==waiting)) //If someone has requested a read/write operation to the bus
	{
		unsigned long long addrRequested = fromBits(m_addrIn,m_addrWidth);
		
		if ((m_minAddrSpaceLimit <= addrRequested) && (addrRequested < m_maxAddrSpaceLimit))
		{
			if (m_ctlIn[rwin] == 0) //if its trying to write to the model
				m_ctlOut[errout].val  = 1; //we send an error
			else
			{
				for (unsigned long long i = 0 ; i< m_bselWidth; i++)
					if (m_bselIn[i])
						for (unsigned long long j=0 ; j<m_byteSize ; j++)
							m_dataOut[i*m_byteSize+j].val = 1;
				m_ctlOut[dtackout].val = 1;
			}
		}
		
		m_state = sending;
	} 
	else if ((m_ctlIn[asin]==0) &&  (m_state==sending))
	{
		m_ctlOut[dtackout].val=0;
		m_ctlOut[errout].val=0;
		m_state = waiting;
	}
	else if (m_state == waiting) {
		//If we don't receive an AS signal we output 0 on dtack
			m_ctlOut[dtackout].val = 0;
			m_ctlOut[errout].val = 0;			
			m_state = waiting;
	}
	
	this->holdIn( active, m_responseTime );		

	return *this;
}

Model & sampleLA::internalFunction( const InternalMessage & msg)
{
	
	this->passivate();
	return *this;
}

Model & sampleLA::outputFunction( const InternalMessage & msg)
{


	//Outputing data over the ports

	for(unsigned i=0; i<NUM_CTL_OUT_PORT; i++)
		if (needSend(m_ctlOut[i]))
			this->sendOutput( msg.time(), *m_ctlOutPorts[i], m_ctlOut[i].val);

	for(unsigned i=0; i<m_addrWidth; i++)	
		if (needSend(m_addrOut[i]))
			this->sendOutput( msg.time(), *m_addrOutPorts[i], m_addrOut[i].val );
				
	for(unsigned i=0; i<m_bselWidth; i++)	
		if (needSend(m_bselOut[i]))
			this->sendOutput( msg.time(), *m_bselOutPorts[i], m_bselOut[i].val );
				
				
	for(unsigned i=0; i<m_dataWidth; i++){
		if (needSend(m_dataOut[i]))
			this->sendOutput( msg.time(), *m_dataOutPorts[i], m_dataOut[i].val );

	}


	return *this;
}
