#include "UC.h"
#include <string>
#include <mainsimu.h>   // MainSimulator::Instance().getParameter( ... )
#include <message.h>

inline bool isFormat2( bit i[] ) { return !i[31] && !i[30]; }
inline bool isFormat3( bit i[] ) { return i[31] && !i[30]; }
inline bool isBranch( bit i[] ) { return isFormat2( i ) && fromBits( i+22, 3 ) == 2; }
inline bool isUnimp( bit i[] ) { return isFormat2( i ) && fromBits( i+22, 3 ) == 0; }
inline bool isArit( bit i[] ) { return isFormat3( i ) && ( !i[24] || fromBits( i+21, 4 ) == 9 ); }
inline bool isRestSave( bit i[] ) { return isFormat3( i ) && fromBits( i+20, 5 ) == 30; }
inline bool isSetHi( bit i[] ) { return isFormat2( i ) && fromBits( i+22, 3 ) == 4; }
inline bool isMul( bit i[] ) { return !i[24] && i[22] && i[20]; }
inline bool isShft( bit i[] ) { return i[24]; }
inline bool isAlu( bit i[] ) { return !isShft( i ) && !isMul( i ); }
inline bool isJmp( bit i[] ) { return isFormat3( i ) && fromBits( i+20, 5 ) == 28; }
inline bool isCall( bit i[] ) { return !i[31] && i[30]; }
inline bool isLoad( bit i[] ) { return i[31] && i[30] && !i[21]; }
inline bool isStore( bit i[] ) { return i[31] && i[30] && i[21]; } 

#define CSEL_O7 15
#define C_MUX_ARES 1
#define C_MUX_IR 2
#define C_MUX_PC 4
#define C_MUX_AL 8
#define MUX_REG 1
#define ALU_ADD 0
#define PC_MUX_ARES 2
#define PC_MUX_ADD 4
#define PC_MUX_INC 1
#define ADDR_MUX_PC 1
#define ADDR_MUX_ARES 0
#define INCDEC_INC 1

static string mportnames[nummports] =
    { "A_SEL", "B_SEL", "C_SEL", "C_MUX", "ALU_FCOD", "PC_MUX", "ALIGNL", "ALIGNS" };

static int mportsizes[nummports] =
    { 5      , 5      , 5      , 4      , 4         , 4       , 4       , 2 };

static string sportnames[numsports] =
{ "C_EN", "C_LATCH_EN", "A_MUX_REG", "A_LATCH_EN", "B_MUX_REG", "B_LATCH_EN", "ENABLE_MUL"
, "ENABLE_ALU", "ENABLE_SHFT", "ALU_RES_EN", "NPC_LATCH_EN", "NPC_EN", "PC_LATCH_EN", "PC_EN"
, "ADDR_MUX", "IR_LATCH_EN", "DATA_IN_EN", "DATA_OUT_EN", "ALIGNL_SIGN", "INCDEC_FCOD", "CWP_LATCH_EN", "CWP_EN"
, "CC_LATCH_EN", "CC_EN", "AS", "RD_WR", "BUSY", "RESET" };

enum { c_I = -1, c_F, c_D, c_E, c_M, c_W };

#define numciclos 6

UC::UC( const string &name )
  : Atomic( name )
  , CCLOGIC( this->addInputPort( "CCLOGIC" ) )
  , CLCK( this->addInputPort( "CLCK" ) )
  , DTACK( this->addInputPort( "DTACK" ) )
  , END( this->addInputPort( "END" ) )
  , off( 0 )
  , end( 0 )
  , waitfmc( 0 )
  , ciclo( c_I )

{
  IR = addPorts( ptInput, "IR", 32, *this );
  PSR = addPorts( ptInput, "PSR", 32, *this );
  
  o_bit **tmportbits[nummports] = { &a_sel, &b_sel, &c_sel, &c_mux, &alu_fcod, &pc_mux, &alignl, &aligns };
  mOUT = new (Port **)[nummports];
  for( int i = 0; i < nummports; i++ ) {
    mportbits[i] = new o_bit[mportsizes[i]];
    *tmportbits[i] = mportbits[i];
    mOUT[i] = addPorts( ptOutput, mportnames[i], mportsizes[i], *this );
  }

  o_bit *tsportbits[numsports] =
  { &c_en, &c_latch_en, &a_mux_reg, &a_latch_en, &b_mux_reg, &b_latch_en, &enable_mul
  , &enable_alu, &enable_shft, &alu_res_en, &npc_latch_en, &npc_en, &pc_latch_en, &pc_en
  , &addr_mux, &ir_latch_en, &data_in_en, &data_out_en, &alignl_sign, &incdec_fcod, &cwp_latch_en, &cwp_en
  , &cc_latch_en, &cc_en, &as, &rd_wr, &busy, &reset };

  sOUT = new (Port *)[numsports];

  for( int i = 0; i < numsports; i++ ) {
    sportbits[i]=tsportbits[i];
    sOUT[i] = & this->addOutputPort( sportnames[i] );
  }

}

UC::~UC() {
  delete [] IR;
  delete [] PSR;
  delete [] sOUT;
}

Model &UC::initFunction()
{
   return *this;
}

Model &UC::externalFunction( const ExternalMessage &msg )
{
  if( off )
    this->passivate();
  else if( msg.port() == CLCK ) {
    if( end && ciclo == c_W ) {
      off = 1;
      this->passivate();
    } else if( ! waitfmc ) {
      ciclo = (ciclo + 1) % numciclos;
      this->holdIn( active, 0 );
    }
  } else if( msg.port() == DTACK ) {
    if( msg.value() == 1 ) {
      waitfmc = 0;
      this->passivate();
    }
  } else if( msg.port() == END ) {
    end = bit( msg.value() );
    this->passivate();
  } else if( msg.port() == CCLOGIC ) {
    cclogic = bit( msg.value() );
    this->passivate();
  } else {
    string portName;
    int portNum;
    nameNum( msg.port().name(), portName, portNum );
    if( portName == "ir" )
      ir[portNum] = bit( msg.value() );
    else
      psr[portNum] = bit( msg.value() );
    this->passivate();
  }

  return *this;
}

Model &UC::internalFunction( const InternalMessage & )
{
  if( as.val )
    waitfmc = 1;
  this->passivate();
  return *this;
}

Model &UC::outputFunction( const InternalMessage &msg )
{
  if( ciclo == c_D && isUnimp( ir ) ) {
    end = 1;
    reset.val = 1;
  }
  if( ( ciclo == c_M || ciclo == c_W ) && isStore( ir ) )
    copyBits( ir+25, a_sel, 5 );
  else
    copyBits( ir+14, a_sel, 5 );
  copyBits( ir, b_sel, 5 );
  if( ciclo != c_F && isCall( ir ) )
    toBits( CSEL_O7, c_sel, 5 );
  else
    copyBits( ir+25, c_sel, 5 );

  c_en.val = ciclo == c_W && !isBranch( ir ) && !isStore( ir );

  if( isArit( ir ) || isRestSave( ir ) )
    toBits( C_MUX_ARES, c_mux, 4 );
  else if( isSetHi( ir ) )
    toBits( C_MUX_IR, c_mux, 4 );
  else if( isLoad( ir ) )
    toBits( C_MUX_AL, c_mux, 4 );
  else
    toBits( C_MUX_PC, c_mux, 4 );
  
  c_latch_en.val = ciclo == c_M;

  a_mux_reg.val = MUX_REG;

  a_latch_en.val = ciclo == c_D;

  b_mux_reg.val = !ir[13];

  b_latch_en.val = ciclo == c_D;

  if( isArit( ir ) ) {
    copyBits( ir+19, alu_fcod, 4 );
    enable_mul.val =  isMul( ir );
    enable_alu.val =  isAlu( ir );
    enable_shft.val =  isShft( ir );
  } else {
    enable_mul.val = 0;
    enable_alu.val = 1;
    enable_shft.val = 0;
    toBits( ALU_ADD, alu_fcod, 4 );
  }

  alu_res_en.val = ciclo == c_E;

  if( isJmp( ir ) )
    toBits( PC_MUX_ARES, pc_mux, 4 );
  else if( isCall( ir ) || isBranch( ir ) && cclogic )
    toBits( PC_MUX_ADD, pc_mux, 4 );
  else
    toBits( PC_MUX_INC, pc_mux, 4 );

  npc_latch_en.val = ciclo == c_M;

  npc_en.val = ciclo == c_W;

  pc_latch_en.val = ciclo == c_M;

  pc_en.val = ciclo == c_W;

  addr_mux.val = ciclo == c_F ? ADDR_MUX_PC : ADDR_MUX_ARES;

  ir_latch_en.val = ciclo == c_F;

  data_in_en.val = ciclo == c_M;

  data_out_en.val = ciclo == c_M;

  copyBits( ir+19, alignl, 2 );
  alignl_sign.val = ir[22];

  copyBits( ir+19, aligns, 2 );

  if( isJmp( ir ) )
    incdec_fcod.val = INCDEC_INC;
  else
    incdec_fcod.val = ir[19];

  cwp_latch_en.val = ciclo == c_E;
  
  if( isJmp( ir ) )
    cwp_en.val = ir[19];
  else
    cwp_en.val = isRestSave( ir );

  cc_latch_en.val = ciclo == c_E;

  cc_en.val = isArit( ir ) && ir[23] && ciclo == c_W;

  busy.val = as.val = ciclo == c_F || ( ciclo == c_M && isLoad( ir ) ) || ( ciclo == c_W && isStore( ir ) );

  rd_wr.val = ciclo == c_F || !isStore( ir );

  for( int i = 0; i < numsports; i++ )
    if( needSend( *sportbits[i] ) )
      this->sendOutput( msg.time(), *sOUT[i], sportbits[i]->val );
  for( int i = 0; i < nummports; i++ )
    for( int j = 0; j < mportsizes[ i ]; j++ )
      if( needSend( mportbits[i][j] ) )
	this->sendOutput( msg.time(), *mOUT[i][j], mportbits[i][j].val );
  return *this;
}
